@@ -556,6 +556,9 @@ jsonsl_feed(jsonsl_t jsn, const jsonsl_char_t *bytes, size_t nbytes)
 JSONSL_API
 const char* jsonsl_strerror(jsonsl_error_t err)
 {
+    if (err == JSONSL_ERROR_SUCCESS) {
+        return "SUCCESS";
+    }
 #define X(t) \
     if (err == JSONSL_ERROR_##t) \
         return #t;
@@ -679,19 +682,21 @@ JSONSL_API
 jsonsl_jpr_t
 jsonsl_jpr_new(const char *path, jsonsl_error_t *errp)
 {
-    char *my_copy;
+    char *my_copy = NULL;
     int count, curidx;
-    struct jsonsl_jpr_st *ret;
-    struct jsonsl_jpr_component_st *components;
+    struct jsonsl_jpr_st *ret = NULL;
+    struct jsonsl_jpr_component_st *components = NULL;
     size_t origlen;
     jsonsl_error_t errstacked;
 
+#define JPR_BAIL(err) *errp = err; goto GT_ERROR;
+
     if (errp == NULL) {
         errp = &errstacked;
     }
 
     if (path == NULL || *path != '/') {
-        *errp = JSONSL_ERROR_JPR_NOROOT;
+        JPR_BAIL(JSONSL_ERROR_JPR_NOROOT);
         return NULL;
     }
 
@@ -703,8 +708,7 @@ jsonsl_jpr_new(const char *path, jsonsl_error_t *errp)
             if (*c == '/') {
                 count++;
                 if (*(c+1) == '/') {
-                    *errp = JSONSL_ERROR_JPR_DUPSLASH;
-                    return NULL;
+                    JPR_BAIL(JSONSL_ERROR_JPR_DUPSLASH);
                 }
             }
         }
@@ -714,7 +718,15 @@ jsonsl_jpr_new(const char *path, jsonsl_error_t *errp)
     }
 
     components = malloc(sizeof(*components) * count);
+    if (!components) {
+        JPR_BAIL(JSONSL_ERROR_ENOMEM);
+    }
+
     my_copy = malloc(strlen(path) + 1);
+    if (!my_copy) {
+        JPR_BAIL(JSONSL_ERROR_ENOMEM);
+    }
+
     strcpy(my_copy, path);
 
     components[0].ptype = JSONSL_PATH_ROOT;
@@ -733,9 +745,7 @@ jsonsl_jpr_new(const char *path, jsonsl_error_t *errp)
         }
 
         if (pathret == JSONSL_PATH_INVALID) {
-            free(components);
-            free(my_copy);
-            return NULL;
+            JPR_BAIL(JSONSL_ERROR_JPR_BADPATH);
         }
     } else {
         curidx = 1;
@@ -744,14 +754,30 @@ jsonsl_jpr_new(const char *path, jsonsl_error_t *errp)
     path--; /*revert path to leading '/' */
     origlen = strlen(path) + 1;
     ret = malloc(sizeof(*ret));
+    if (!ret) {
+        JPR_BAIL(JSONSL_ERROR_ENOMEM);
+    }
+    ret->orig = malloc(origlen);
+    if (!ret->orig) {
+        JPR_BAIL(JSONSL_ERROR_ENOMEM);
+    }
     ret->components = components;
     ret->ncomponents = curidx;
     ret->basestr = my_copy;
-    ret->orig = malloc(origlen);
     ret->norig = origlen-1;
     strcpy(ret->orig, path);
 
     return ret;
+
+    GT_ERROR:
+    free(my_copy);
+    free(components);
+    if (ret) {
+        free(ret->orig);
+    }
+    free(ret);
+    return NULL;
+#undef JPR_BAIL
 }
 
 void jsonsl_jpr_destroy(jsonsl_jpr_t jpr)
@@ -764,7 +790,7 @@ void jsonsl_jpr_destroy(jsonsl_jpr_t jpr)
 
 JSONSL_API
 jsonsl_jpr_match_t
-jsonsl_jpr_match(jsonsl_jpr_t jpr,
+jsonsl_jpr_match(const jsonsl_jpr_t jpr,
                    jsonsl_type_t parent_type,
                    unsigned int parent_level,
                    const char *key,
@@ -798,17 +824,22 @@ jsonsl_jpr_match(jsonsl_jpr_t jpr,
     }
 
     /* Check numeric array index */
-    if (p_component->ptype == JSONSL_PATH_NUMERIC
-            && parent_type == JSONSL_T_LIST) {
-        if (p_component->idx != nkey) {
-            return JSONSL_MATCH_NOMATCH;
-        } else {
-            if (parent_level == jpr->ncomponents-1) {
-                return JSONSL_MATCH_COMPLETE;
+    if (p_component->ptype == JSONSL_PATH_NUMERIC) {
+        if (parent_type == JSONSL_T_LIST) {
+            if (p_component->idx != nkey) {
+                return JSONSL_MATCH_NOMATCH;
             } else {
-                return JSONSL_MATCH_POSSIBLE;
+                if (parent_level == jpr->ncomponents-1) {
+                    return JSONSL_MATCH_COMPLETE;
+                } else {
+                    return JSONSL_MATCH_POSSIBLE;
+                }
             }
+        } else if (p_component->is_arridx) {
+            return JSONSL_MATCH_TYPE_MISMATCH;
         }
+    } else if (p_component->is_arridx && parent_type == JSONSL_T_OBJECT) {
+        return JSONSL_MATCH_TYPE_MISMATCH;
     }
 
     /* Check lengths */
@@ -818,9 +849,7 @@ jsonsl_jpr_match(jsonsl_jpr_t jpr,
 
     /* Check string comparison */
     cmpret = strncmp(p_component->pstr, key, nkey);
-    if (cmpret != 0) {
-        return JSONSL_MATCH_NOMATCH;
-    } else {
+    if (cmpret == 0) {
         if (parent_level == jpr->ncomponents-1) {
             return JSONSL_MATCH_COMPLETE;
         } else {
@@ -828,8 +857,6 @@ jsonsl_jpr_match(jsonsl_jpr_t jpr,
         }
     }
 
-    /* Never reached, but make the compiler happy */
-    abort();
     return JSONSL_MATCH_NOMATCH;
 }
 
